==> externalVector_1.0.3.tar.gz: externalVector/R/allocators.R <==
##
## Memory allocators for external resources.
##

setClass("externalAllocator")

setClass("gcAllocator",
         contains="externalAllocator")

##
## Base class for external resource allocated by the allocators
##

setClass("externalResource")

setMethod("initialize",
          signature(.Object = "externalResource"),
          function (.Object, ...)
          allocate(.Object, ...))

##
## Return a suitable allocator for the resource
##
if (!isGeneric("allocator"))
    setGeneric("allocator",
               function(resource)
               standardGeneric("allocator"))

##
## Each resource type should define its default allocator
##
setVirtualMethod("allocator",
          signature(resource="externalResource"))

if (!isGeneric("allocatedType"))
    setGeneric("allocatedType",
               function(resource)
               standardGeneric("allocatedType"))

setMethod("allocatedType",
          signature("externalResource"),
          function (resource) NULL)

if (!isGeneric("allocatedSize"))
    setGeneric("allocatedSize",
               function(resource)
               standardGeneric("allocatedSize"))

setVirtualMethod("allocatedSize",
                 signature(resource="externalResource"))

##
## Allocate some raw memory for the resource using allocator alloc,
## create an external pointer with that raw memory, initialize the
## resource with the pointer and return it. If type is a vector type
## (one of logical, integer, numeric, complex, character or list) then
## size is the length of the vector. Otherwise, size is in bytes.
##
## The allocated memory is not initialized.
##
if (!isGeneric("allocate"))
    setGeneric("allocate",
               function(resource, alloc, size, type, ...)
               standardGeneric("allocate"))

setVirtualMethod("allocate",
                 signature(alloc="externalAllocator"),
                 signature(resource="externalResource"))

setMethod("allocate",
          signature(resource="externalResource", alloc="missing"),
          function (resource, alloc, size, type, ...)
      {
          alloc <- allocator(resource)
          if (missing(size))
              size <- allocatedSize(resource)
          if (missing(type))
              type <- allocatedType(resource)
          allocate(resource, alloc=alloc, size=size, type=type, ...)
      })

##
## This can be called explicitly to (potentially) free the memory
## 
if (!isGeneric("deallocate"))
    setGeneric("deallocate",
               function(resource, alloc)
               standardGeneric("deallocate"))

setVirtualMethod("deallocate",
                 signature(alloc="externalAllocator"),
                 signature(resource="externalResource"))

setMethod("deallocate",
          signature(resource="externalResource",
                    alloc="missing"),
          function (resource, alloc)
      {
          deallocate(resource, alloc=allocator(resource))
      })

##
## Given a resource and an external pointer allocated for that
## resource, initialize and return the resource.
##
if (!isGeneric("initializeResource"))
    setGeneric("initializeResource",
               function(resource, ptr, size, type, ...)
               standardGeneric("initializeResource"))

setVirtualMethod("initializeResource",
                 signature(resource="externalResource"),
                 signature(ptr="externalptr"))

##
## Retrieve the external pointer associated with the resource
##
if (!isGeneric("getPointer"))
    setGeneric("getPointer",
               function(resource)
               standardGeneric("getPointer"))

setVirtualMethod("getPointer",
                 signature(resource="externalResource"))

if (!isGeneric("reinitializePointer"))
    setGeneric("reinitializePointer",
               function(resource, alloc)
               standardGeneric("reinitializePointer"))

setVirtualMethod("reinitializePointer",
                 signature(alloc="externalAllocator"),
                 signature(resource="externalResource"))

setMethod("reinitializePointer",
          signature(resource="externalResource",
                    alloc="missing"),
          function (resource, alloc) 
          reinitializePointer(resource, alloc=allocator(resource)))

if (!isGeneric("external.size"))
    setGeneric("external.size",
               function(resource, alloc)
               standardGeneric("external.size"))

setVirtualMethod("external.size",
                 signature(alloc="externalAllocator"),
                 signature(resource="externalResource"))

setMethod("external.size",
          signature(resource="externalResource",
                    alloc="missing"),
          function (resource, alloc) 
          external.size(resource, alloc=allocator(resource)))

if (!isGeneric("external.size<-"))
    setGeneric("external.size<-",
               function(resource,
                        copy = TRUE, alloc,
                        value)
               standardGeneric("external.size<-"))

setVirtualMethod("external.size<-",
                 signature(alloc="externalAllocator"),
                 signature(resource="externalResource",
                           copy="logical"))

setReplaceMethod("external.size",
                 signature(resource = "externalResource",
                           alloc = "missing"),
                 function (resource, copy, alloc, value) 
             {
                 if (missing(copy))
                     copy <- TRUE
                 "external.size<-"(resource, copy,
                                   allocator(resource), value)
             })

##
## Methods for gcAllocator
##
setMethod("allocate",
          signature(resource="externalResource",
                    alloc="gcAllocator"),
          function(resource, alloc, size, type, ...)
      {
          ptr <- .Call("gcExternalPtr_new", size, type,
                       PACKAGE="externalVector")
          initializeResource(resource, ptr, size, type, ...)
      })

setMethod("reinitializePointer",
          signature(resource="externalResource",
                    alloc="gcAllocator"),
          function (resource, alloc) 
      {
          .Call("gcExternalPtr_restore", getPointer(resource),
                PACKAGE="externalVector")
          resource
      })

setMethod("external.size",
          signature(resource="externalResource",
                    alloc="gcAllocator"),
          function(resource, alloc)
      {
          .Call("gcExternalPtr_size", getPointer(x),
                PACKAGE="externalVector")
      })

setReplaceMethod("external.size",
                 signature(resource="externalResource",
                           copy = "logical",
                           alloc = "gcAllocator",
                           value="numeric"),
                 function(resource, copy, alloc, value)
             {
                 .Call("gcExternalPtr_resize", getPointer(resource),
                       value, copy,
                       PACKAGE="externalVector")
                 resource
             })

setMethod("deallocate",
          signature(resource="externalResource",
                    alloc="gcAllocator"),
          function(resource, alloc)
      {
          .Call("gcExternalPtr_delete", getPointer(resource))
          invisible(NULL)
      })

==> externalVector_1.4.0.tar.gz: externalVector/R/allocators.R <==
##
## Memory allocators for external resources.
##

setClass("externalAllocator")

setClass("gcAllocator",
         contains="externalAllocator")

##
## Base class for external resource allocated by the allocators
##

setClass("externalResource")

setMethod("initialize",
          signature(.Object = "externalResource"),
          function (.Object, ...)
          allocate(.Object, ...))

##
## Return a suitable allocator for the resource
##
setGeneric("allocator",
               function(resource)
               standardGeneric("allocator"))

##
## Each resource type should define its default allocator
##
setVirtualMethod("allocator",
          signature(resource="externalResource"))

setGeneric("allocatedType",
               function(resource)
               standardGeneric("allocatedType"))

setMethod("allocatedType",
          signature("externalResource"),
          function (resource) NULL)

setGeneric("allocatedSize",
               function(resource)
               standardGeneric("allocatedSize"))

setVirtualMethod("allocatedSize",
                 signature(resource="externalResource"))

##
## Allocate some raw memory for the resource using allocator alloc,
## create an external pointer with that raw memory, initialize the
## resource with the pointer and return it. If type is a vector type
## (one of logical, integer, numeric, complex, character or list) then
## size is the length of the vector. Otherwise, size is in bytes.
##
## The allocated memory is not initialized.
##
setGeneric("allocate",
               function(resource, alloc, size, type, ...)
               standardGeneric("allocate"))

setVirtualMethod("allocate",
                 signature(alloc="externalAllocator"),
                 signature(resource="externalResource"))

setMethod("allocate",
          signature(resource="externalResource", alloc="missing"),
          function (resource, alloc, size, type, ...)
      {
          alloc <- allocator(resource)
          if (missing(size))
              size <- allocatedSize(resource)
          if (missing(type))
              type <- allocatedType(resource)
          allocate(resource, alloc=alloc, size=size, type=type, ...)
      })

##
## This can be called explicitly to (potentially) free the memory
##
setGeneric("deallocate",
               function(resource, alloc)
               standardGeneric("deallocate"))

setVirtualMethod("deallocate",
                 signature(alloc="externalAllocator"),
                 signature(resource="externalResource"))

setMethod("deallocate",
          signature(resource="externalResource",
                    alloc="missing"),
          function (resource, alloc)
      {
          deallocate(resource, alloc=allocator(resource))
      })

##
## Given a resource and an external pointer allocated for that
## resource, initialize and return the resource.
##
setGeneric("initializeResource",
               function(resource, ptr, size, type, ...)
               standardGeneric("initializeResource"))

setVirtualMethod("initializeResource",
                 signature(resource="externalResource"),
                 signature(ptr="externalptr"))

##
## Retrieve the external pointer associated with the resource
##
setGeneric("getPointer",
               function(resource)
               standardGeneric("getPointer"))

setVirtualMethod("getPointer",
                 signature(resource="externalResource"))

setGeneric("reinitializePointer",
               function(resource, alloc)
               standardGeneric("reinitializePointer"))

setVirtualMethod("reinitializePointer",
                 signature(alloc="externalAllocator"),
                 signature(resource="externalResource"))

setMethod("reinitializePointer",
          signature(resource="externalResource",
                    alloc="missing"),
          function (resource, alloc)
          reinitializePointer(resource, alloc=allocator(resource)))

setGeneric("external.size",
               function(resource, alloc)
               standardGeneric("external.size"))

setVirtualMethod("external.size",
                 signature(alloc="externalAllocator"),
                 signature(resource="externalResource"))

setMethod("external.size",
          signature(resource="externalResource",
                    alloc="missing"),
          function (resource, alloc)
          external.size(resource, alloc=allocator(resource)))

setGeneric("external.size<-",
               function(resource,
                        copy = TRUE, alloc,
                        value)
               standardGeneric("external.size<-"))

setVirtualMethod("external.size<-",
                 signature(alloc="externalAllocator"),
                 signature(resource="externalResource",
                           copy="logical"))

setReplaceMethod("external.size",
                 signature(resource = "externalResource",
                           alloc = "missing"),
                 function (resource, copy, alloc, value)
             {
                 if (missing(copy))
                     copy <- TRUE
                 "external.size<-"(resource, copy,
                                   allocator(resource), value)
             })

##
## Methods for gcAllocator
##
setMethod("allocate",
          signature(resource="externalResource",
                    alloc="gcAllocator"),
          function(resource, alloc, size, type, ...)
      {
          ptr <- .Call("gcExternalPtr_new", size, type,
                       PACKAGE="externalVector")
          initializeResource(resource, ptr, size, type, ...)
      })

setMethod("reinitializePointer",
          signature(resource="externalResource",
                    alloc="gcAllocator"),
          function (resource, alloc)
      {
          .Call("gcExternalPtr_restore", getPointer(resource),
                PACKAGE="externalVector")
          resource
      })

setMethod("external.size",
          signature(resource="externalResource",
                    alloc="gcAllocator"),
          function(resource, alloc)
      {
          .Call("gcExternalPtr_size", getPointer(x),
                PACKAGE="externalVector")
      })

setReplaceMethod("external.size",
                 signature(resource="externalResource",
                           copy = "logical",
                           alloc = "gcAllocator",
                           value="numeric"),
                 function(resource, copy, alloc, value)
             {
                 .Call("gcExternalPtr_resize", getPointer(resource),
                       value, copy,
                       PACKAGE="externalVector")
                 resource
             })

setMethod("deallocate",
          signature(resource="externalResource",
                    alloc="gcAllocator"),
          function(resource, alloc)
      {
          .Call("gcExternalPtr_delete", getPointer(resource),
                PACKAGE="externalVector")
          invisible(NULL)
      })
